#pragma once

#include <cassert>
#include <Qt>
#include <QObject>
#include <QImage>
#include <QColor>
#include <bgtu/computer_graphics_lab_work/renderer_api/color.hpp>
#include <bgtu/computer_graphics_lab_work/renderer_api/point.hpp>
#include <bgtu/computer_graphics_lab_work/renderer_api/voxel_drawer.hpp>
#include <bgtu/computer_graphics_lab_work/renderer_api/sprite.hpp>

namespace BGTU::ComputerGraphicsLabWork::QtUtilities {
	template<class sprite_data_t>
	class OwnedQImage : public QImage, public RendererApi::VoxelDrawer::Exporter<sprite_data_t> {
	private:
		RendererApi::Sprite<sprite_data_t> const **owners;

	public:
		OwnedQImage(unsigned width, unsigned height) : QImage(width, height, QImage::Format_ARGB32), owners{new RendererApi::Sprite<sprite_data_t> const *[width * height]} {
			for (RendererApi::Sprite<sprite_data_t> const **p = this->owners - 1; p-- >= this->owners;) {
				*p = nullptr;
			}
		}

		OwnedQImage() : QImage(), owners{nullptr} {}


		void setPixelOwner(unsigned x, unsigned y, RendererApi::Sprite<sprite_data_t> const *o) {
			assert(x < this->width());
			assert(y < this->height());

			this->owners[y * this->width() + x] = o;
		}

		RendererApi::Sprite<sprite_data_t> const *getPixelOwner(unsigned x, unsigned y) const {
			assert(x < this->width());
			assert(y < this->height());

			return this->owners[y * this->width() + x];
		}

		void set_pixel(RendererApi::PointI<2>::component_t x, RendererApi::PointI<2>::component_t y, RendererApi::Color c, RendererApi::Sprite<sprite_data_t> const *owner) final {
			this->setPixelColor(x, y, QColor{c.red, c.green, c.blue});
			this->setPixelOwner(x, y, owner);
		}

		~OwnedQImage() override {
			if (this->owners != nullptr)
				delete[] this->owners;
		}
	};
}